from django.utils.translation import gettext_lazy as _

from gdaps.api import PluginConfig

from . import __version__


class CorePluginMeta:
    verbose_name = _("MedUX Core")
    author = "Christian González"
    author_email = "christian.gonzalez@nerdocs.at"
    vendor = "nerdocs"
    description = _("Medux Core Plugin")
    category = _("Core")
    visible = True
    version = __version__


class CoreConfig(PluginConfig):
    """MedUX Core Plugin"""

    name = "medux.core"
    PluginMeta = CorePluginMeta

    def ready(self):
        # This function is called after the app and all models are loaded.
        #
        # You can do some initialization here, but beware: it should rather
        # return fast, as it is called at each Django start, even on
        # management commands (makemigrations/migrate etc.).
        #
        # Avoid interacting with the database especially 'save' operations,
        # if you don't *really* have to."""

        try:
            from . import signals
        except ImportError:
            pass



import graphene
from .models import User
from graphene_django import DjangoObjectType

from gdaps.graphene.api import IGrapheneSchema


class UserType(DjangoObjectType):
    """MedUX's built-in User type"""

    class Meta:
        model = User


class UserQuery:
    users = graphene.List(UserType)

    @staticmethod
    def resolve_users(self, info, **kwargs):
        return User.objects.all()


# Here comes the magic:
class UserSchema(IGrapheneSchema):
    query = UserQuery



from django.contrib.auth.views import LoginView as DjangoLoginView
from django.http import HttpRequest
from django.shortcuts import render, redirect
from django.contrib.auth import authenticate, login
from django.utils.translation import gettext_lazy as _
from django.views.generic.base import ContextMixin

from .forms import LoginForm, SignUpForm


class LoginView(DjangoLoginView):
    template_name = "registration/login.html"
    # form_class = LoginForm
    message = ""
    extra_context = {"message": message}

    # def get(self, request, *args, **kwargs):
    #     form = self.form_class(initial=self.extra_context)
    #     return render(
    #         request,
    #         self.template_name,
    #         {"form": form, "message": self.message},
    #     )


#
#     def post(self, request: HttpRequest, *args, **kwargs):
#         form = self.form_class(request.POST)
#         if form.is_valid():
#             username = form.cleaned_data.get("username")
#             password = form.cleaned_data.get("password")
#             user = authenticate(username=username, password=password)
#             login(request, user)
#             if user is not None:
#                 login(request, user)
#                 return redirect("")
#             else:
#                 self.message = _("Nope")
#         else:
#             self.message = _("Error validating toe login form")


def register_user(request):

    msg = None
    success = False

    if request.method == "POST":
        form = SignUpForm(request.POST)
        if form.is_valid():
            form.save()
            username = form.cleaned_data.get("username")
            raw_password = form.cleaned_data.get("password1")
            user = authenticate(username=username, password=raw_password)

            msg = "User created"
            success = True

            # return redirect("/login/")

        else:
            msg = "Form is not valid"
    else:
        form = SignUpForm()

    return render(
        request,
        "accounts/user-add.html",
        {"form": form, "msg": msg, "success": success},
    )



from django import template

register = template.Library()


@register.inclusion_tag("alerts/alert.html")
def toast(title, message, badge=None, icon=None, tag="information", autoclose=False):
    """

    :param context : a dict with the following keys:
        title: the title of the  toast
        message: the message to display
        icon: the icon name to display
        badge: an optional badge
        tag:  one of primary, secondary, light, dark, info, danger, warning, error, critical
        autoclose: if given, the alert will close automatically
    """
    return {
        "title": title,
        "message": message,
        "icon": icon,
        "badge": badge,
        "tag": tag,
        "light_tags": [
            "light",
            "info",
            "warning",
        ],
        "dark_tags": [
            "primary",
            "secondary",
            "danger",
            "dark",
        ],
        "autoclose": bool(autoclose),
    }



from django import forms
from django.contrib.auth.forms import UserCreationForm
from django.contrib.auth.models import User


class LoginForm(forms.Form):
    def __init__(self, request, *args, **kwargs):
        super().__init__(*args, **kwargs)

    username = forms.CharField(
        widget=forms.TextInput(attrs={"placeholder": "Username"})
    )
    password = forms.CharField(
        widget=forms.PasswordInput(
            attrs={"placeholder": "Password", "class": "form2-control"}
        )
    )


class SignUpForm(UserCreationForm):
    username = forms.CharField(
        widget=forms.TextInput(
            attrs={"placeholder": "Username", "class": "form-control"}
        )
    )
    email = forms.EmailField(
        widget=forms.EmailInput(attrs={"placeholder": "Email", "class": "form-control"})
    )
    password1 = forms.CharField(
        widget=forms.PasswordInput(
            attrs={"placeholder": "Password", "class": "form-control"}
        )
    )
    password2 = forms.CharField(
        widget=forms.PasswordInput(
            attrs={"placeholder": "Password check", "class": "form-control"}
        )
    )

    class Meta:
        model = User
        fields = ("username", "email", "password1", "password2")



# import requests
# from django import http
# from django.conf import settings
# from django.template import engines
from django.views.generic import TemplateView


# def catchall_dev(request, upstream="http://localhost:8080"):
#     upstream_url = upstream + request.path
#     response = requests.get(upstream_url, stream=True)
#     content_type = response.headers.get("Content-Type")
#
#     if content_type == "text/html; charset=UTF-8":
#         return http.HttpResponse(
#             content=engines["django"].from_string(response.text).render(),
#             status=response.status_code,
#             reason=response.reason,
#         )
#
#     else:
#         return http.StreamingHttpResponse(
#             streaming_content=response.iter_content(2 ** 12),
#             content_type=content_type,
#             status=response.status_code,
#             reason=response.reason,
#         )


catchall = TemplateView.as_view(template_name="layouts/base.html")

# catchall = catchall_dev if settings.DEBUG else catchall_prod



from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from menu import Menu

from medux.core.api import IViewMode
from medux.core.api.menus import MenuItem, MenuSeparator

__all__ = ["menu_patient", "menu_extras__settings", "menu_notifications"]


class SearchViewMode(IViewMode):
    title = _("Search")
    url = "/search"
    icon = "bi-search"
    weight = 20


# Patient menu
_menu_patient__new = MenuItem(
    title=_("New"),
    url=reverse("home"),
    weight=0,
    icon="bi-person-circle",
)
menu_patient = MenuItem(
    title=_("Patient"),
    url=reverse("home"),
    weight=0,
    children=[
        _menu_patient__new,
        MenuSeparator(),
    ],
)
Menu.add_item("main_menu", menu_patient)

# Extras
menu_extras__settings = MenuItem(
    _("Settings"),
    reverse("home"),
    slug="settings",
    weight=10,
    icon="bi-gear",
    icon_only=True,
)
menu_extras = MenuItem(
    _("Extras"), url=reverse("home"), weight=30, children=[menu_extras__settings]
)
Menu.add_item("main_menu", menu_extras)


# top right menu
menu_notifications = MenuItem(
    _("Notifications"),
    url=reverse("home"),
    weight=20,
    icon="bi-bell",
    icon_only=True,
    badge=True,  # FIXME: this shouldn't be hardcoded here
)
Menu.add_item(
    "top_navbar",
    menu_notifications,
)


Menu.add_item(
    "top_navbar",
    MenuItem(
        title=lambda request: request.user,
        url=reverse("home"),
        slug="myaccount",
        weight=99,
        icon="bi-user",
        children=[
            MenuItem("Edit Profile", url=reverse("home"), icon="bi-user"),
            MenuItem(
                title="Admin",
                url=reverse("admin:index"),
                # check=lambda request: request.user.is_superuser,
            ),
            MenuSeparator(),
            MenuItem(
                title=_("Logout"),
                url=reverse("logout"),
                icon="bi-box-arrow-right",
            ),
        ],
    ),
)



import os

# from django.conf import settings
from django.utils.translation import gettext_lazy as _

from gdaps.api import PluginConfig

# from gdaps.frontend.conf import frontend_settings
from . import __version__


class AlertsPluginMeta:
    """This configuration is the introspection data for plugins."""

    # the plugin machine "name" is taken from the AppConfig, so no name here
    verbose_name = _("Alerts")
    author = "Christian González"
    author_email = "christian.gonzalez@nerdocs.at"
    vendor = "Nerdocs"
    description = _("MedUX Alerts plugin")
    category = _("Core")
    visible = True
    version = __version__
    # compatibility = "medux.core>=2.3.0"


class AlertsConfig(PluginConfig):
    """A GDAPS Django app plugin.

    It needs a special parameter named ``PluginMeta``. It is the key for GDAPS
    to recognize this app as a GDAPS plugin.
    ``PluginMeta`` must point to a class that implements certain attributes
    and methods.
    """

    # Dotted Python path to the application, e.g. '/home/christian/Projekte/medux/medux/plugins.fooplugin'
    # It has to start with '/home/christian/Projekte/medux/medux/plugins' to be recognized as plugin.
    name = "medux.alerts"

    # This is the most important attribute of a GDAPS plugin app.
    PluginMeta = AlertsPluginMeta

    def ready(self):
        try:
            from . import signals
        except ImportError:
            pass

        # import implementations of global interfaces
        from . import impl



from django.db import models
from django.contrib.auth.models import AbstractUser

# from django.utils.translation import gettext_lazy as _
from django.urls import reverse
from django.utils.translation import gettext_lazy as _


class User(AbstractUser):
    """Default user model for MedUX."""

    class Meta:
        permissions = [("can_login", _("Can login"))]

    def get_absolute_url(self):
        """Get url for user's detail view.

        Returns:
            str: URL for user detail.

        """
        return reverse("users:detail", kwargs={"username": self.username})



from django.test.signals import setting_changed
from gdaps.conf import PluginSettings

NAMESPACE = "CORE"

DEFAULTS = {}
IMPORT_STRINGS = ()
REMOVED_SETTINGS = ()

core_settings = PluginSettings(NAMESPACE, DEFAULTS, IMPORT_STRINGS)


def reload_core_settings(*args, **kwargs):
    setting = kwargs["setting"]
    if setting == "CORE":
        core_settings.reload()


setting_changed.connect(reload_core_settings)



# from django.contrib.auth import get_user_model
# User = get_user_model()
from django.contrib.auth.mixins import PermissionRequiredMixin, LoginRequiredMixin
from django.utils.translation import gettext_lazy as _
from django.views.generic import TemplateView

from medux.core.api import IViewMode, IGlobalJavascript


class MeduxBaseMixin(PermissionRequiredMixin):
    """Mixin for all views that MedUX uses.

    Inherit from this view if you want to create a new page etc. in MedUX.
    You have to specify the `permission_required`attribute, since all MedUX views
    need to have a permission for seeing it.
    """

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(
            {
                "viewmodes": iter(IViewMode),
                "global_javascripts": iter(IGlobalJavascript),
            }
        )
        return context


class HomeView(MeduxBaseMixin, TemplateView):
    template_name = "home.html"
    # FIXME find a correct permission for home, and use MeduxBaseMixin
    permission_required = ()
    extra_context = {
        "viewmodes": iter(IViewMode),
    }



from .interfaces import *
from django.db import models
from .interfaces import *


class CreatedUpdatedMixin(models.Model):
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True



from gdaps.api import Interface


@Interface
class IViewMode:
    """Interface for MedUX ViewModes.

    A ViewMode provides data and methods for displaying an icon for the
    main viewing modes of MedUX.
    """

    __service__ = True

    #: The title which is displayed when hovering over the icon.
    title = ""

    #: the URL that should be called. Can be a Dajngo reverse() URL.
    url = ""

    #: the icon as text, using Bootstrap Icons
    icon = "bi-file-earmark"

    #: icon_type determines where the icon is fetched from:
    #: bi (Bootstrap icon) or png, then it is loaded from static files
    icon_type = "bi"  # png

    #: the weight of the icon in the list. The higher the weight, the "deeper" the icon.
    weight = 0


@Interface
class IGlobalJavascript:
    """Interface for adding Js file to a global scope.

    The given file will be loaded in the global context and is then available to
    all other plugins too, in every loaded page.
    Be aware just to add Js code that is small and fast, to not blow up the application
    as whole.

    You have to specify the (relative to static dir) file path of the Js script in the
    `file` attribute.
    """

    __service__ = True

    #: The (relative to static dir) file path of the Js script
    file: str = ""


@Interface
class IRestRouter:
    """Interface for collecting Django Rest Framework routers under one global URL.

    Just provide an URL and the viewset, and MedUX registers this router into the
    DefaultRouter.
    """

    url = ""
    viewset = None



from django.urls import resolve
from menu import MenuItem as DjangoMenuItem


class MenuItem(DjangoMenuItem):
    """Custom MenuItem that checks permissions"""

    # def __init__(self, icon="", *args, **kwargs):
    #     super().__init__(*args, **kwargs)
    #     self.icon = icon

    def check(self, request):
        """Check permissions based on our view"""
        is_visible = True
        match = resolve(self.url)

        # do something with match, and possibly change is_visible...
        # if self.permission:
        #     self.visible = user.has_perm(self.permission)

        self.visible = is_visible

    def __str__(self):
        return self.title


class MenuSeparator(DjangoMenuItem):
    def __init__(self, *args, **kwargs):
        super().__init__(title="", url="", separator=True, *args, **kwargs)



from django.urls import path, re_path, include
from django.contrib.auth import views

from django.contrib.auth.views import LoginView

from .views import HomeView

app_name = "core"

root_urlpatterns = [
    path("", HomeView.as_view(), name="home"),
    path("accounts/", include("django.contrib.auth.urls")),
]

# Django login accounts views:
# accounts/login/ [name='login']
# accounts/logout/ [name='logout']
# accounts/password_change/ [name='password_change']
# accounts/password_change/done/ [name='password_change_done']
# accounts/password_reset/ [name='password_reset']
# accounts/password_reset/done/ [name='password_reset_done']
# accounts/reset/<uidb64>/<token>/ [name='password_reset_confirm']
# accounts/reset/done/ [name='password_reset_complete']
